using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using System.Net;
using System.Text.RegularExpressions;

namespace LibJasinski.Network
{
	public class WebScraper
	{
		private InternetConnector _conn;
		private string _proxyServer;
		private string _html;
		private MatchCollection _mc;

		public event NoArgsDelegate ProxyStateChanged;

		public const string NotApplicable = "(n/a)";

		public WebScraper()
		{
			_conn = InternetConnector.Default; // new InternetConnector();
			_conn.ProxyStateChanged += new NoArgsDelegate(_conn_ProxyStateChanged);
		}

		private void _conn_ProxyStateChanged()
		{
			if (this.ProxyStateChanged != null)
				this.ProxyStateChanged();
		}

		public string ProxyServer
		{
			get { return _proxyServer; }
		}

		public ProxyState ProxyState
		{
			get { return _conn.ProxyState; }
		}

		public string Html
		{
			get { return _html; }
		}

		public MatchCollection Matches
		{
			get { return _mc; }
		}

		protected bool Get(string sourceUrl)
		{
			bool result = false;
			bool tryAgain = true;
			WebClient wc;
			//string html = "";

            try
            {
				while (tryAgain)
				{
					wc = _conn.GetConnection();

					if (_conn.ProxyState == ProxyState.NotUsing)
						_proxyServer = NotApplicable;
					else
						_proxyServer = _conn.ProxyServer;

					if (wc != null)
					{
						try
						{
							_html = wc.DownloadString(sourceUrl + "?zzz=" + Guid.NewGuid().ToString());
							_conn.ProxyState = ProxyState.Verified;
							tryAgain = false;
							result = true;
						}
						catch (Exception ex)
						{
							Debug.Print(ex.Message);
							if (_conn.ProxyState == ProxyState.NotUsing)
								tryAgain = false;
						}
					}
				}


				//if (html.Length > 0)
				//{
				//    string pattern = Cfg.Def.RegexPattern;
				//    Regex regex = new Regex(pattern);
				//    MatchCollection mc = regex.Matches(html);
				//    int n = mc.Count;
				//    if (n > 0)
				//    {
				//        _verse = InsertLineBreaks(mc[0].Groups["verse"].Value);
				//        _reference = mc[0].Groups["reference"].Value;
				//        result = _verse.Length > 0 & _reference.Length > 0;
				//    }
				//}
				return result;
            }
            catch (Exception ex)
            {
				Debug.Print(ex.Message);
                return false;
            }
		}

		protected bool Get(string sourceUrl, string pattern)
		{
			bool result = false;
			
			if (this.Get(sourceUrl))
			{
				string html = this.Html;

				if (html.Length > 0)
				{
					Regex regex = new Regex(pattern);
					_mc = regex.Matches(html);
					result = (_mc != null);
				}
			}

			return result;
		}
	}
}
